#include <Servo.h>

Servo myservo;  // create servo object to control a servo

unsigned int pressure_signal = A1; // pin 1 associated with the pressure signal from FSR sensor
unsigned int s_current = A2; // pin 2 associated with the current signal form ACS712
const int vibration = 9; //pin for vibration motor
int step = 10;
const unsigned int ms_sampling = 100;  //sampling time 
const unsigned int ms_total_time_recording= 2500; //total time recording for calibration
int tot_samples = ms_total_time_recording/ ms_sampling;
int pmax = 0;
int pmin = 0;
int range = 0;
int mean_current= 0;

// mean current at rest
void record_offset_current(){ 
   int sum_current = 0;
   int count = 0;
   for (count = 0; count < tot_samples; count++ ){    
      int current_rest = analogRead(s_current);
      
      sum_current += current_rest;
      _delay_ms (ms_sampling);
    }   
    mean_current= (int) sum_current / tot_samples;         
}


// mean pressure on FSR at rest 
void record_min_pressure(){
    _delay_ms(2000);
    int sum_value = 0;
    int count = 0;
    for (count = 0; count < tot_samples; count++ ){      
      int read_pressure = analogRead(pressure_signal);      
      sum_value += read_pressure; 
      _delay_ms (ms_sampling);
    }    
    pmin = (int) sum_value / tot_samples;    
  }


// mean pressure on FSR during muscle contraction
void record_max_pressure(){
   _delay_ms(2000);
    int sum_value = 0;
    int count = 0;
    for (count = 0; count < tot_samples; count++ ){      
      int read_pressure = analogRead(pressure_signal);      
      sum_value += read_pressure;    
      _delay_ms(ms_sampling);
    }
    pmax = (int) sum_value / tot_samples;    
  }


// function computing the current servomotor rotor position
int servomotor_positioning(){
  unsigned int pcurr = analogRead(pressure_signal);
  Serial.println("Pcurr: ");
  Serial.println(pcurr);   
  int out =  (pcurr-pmin); // working range
  int position = 5; // for avoiding the hum produced by the servomotor when positioned at 0 degrees

  switch(out){
     case 0:
        position = 5; //angle 5°  
     break;

     case 1:
        position = 18; //angle 18°
     break;

     case 2:
        position = 36; //angle 36°
     break;

     case 3:
        position = 54; //angle 54°
     break;

     case 4:
        position = 72; //angle 72°
     break;

     case 5:
        position = 90; //angle 90°
     break;

     case 6:
        position = 108; //angle 108°
     break;

     case 7:
        position = 126; //angle 126°
     break;

     case 8:
        position = 144; //angle 144°
     break;

     case 9:
        position = 162; //angle 162°
     break;

     case 10:
        position = 180; //angle 180°
     break;
   
     default:
        position = 5; //angle 0°
     break;   
   }
  return position;  
  }


void setup() {
  // put your setup code here, to run once:
   Serial.begin( 9600 ); //serial comunication 9600 bit/s
   myservo.attach(5);  // attaches the servo on pin 5 to the servo object   
   myservo.write(5); // 5 degree position for avoiding the hum produced by the servomotor
   pinMode(vibration, OUTPUT); //declaring the PIN as output for controlling the vibration motor 
 
   record_offset_current(); // colling the function

// (at start up system) a 1st vibration is provided to the user to warn him to push the button and stay at rest
    digitalWrite(vibration, HIGH);
    _delay_ms(1000); 
    digitalWrite(vibration, LOW); 
    _delay_ms(500); 
 
    record_min_pressure(); //calling the function for recording the rest condition
     
// a 2nd vibration is provided to the user to warn him to push the button and contract the muscles
    digitalWrite(vibration, HIGH);
    _delay_ms(1000); 
    digitalWrite(vibration, LOW); 
    _delay_ms(500); 

    record_max_pressure();  // calling the function for recording the muscle contraction condition
    
// a 3rd vibration is provided to the user to warn him that the system calibration is ended
    digitalWrite(vibration, HIGH);
    _delay_ms(1000); 
    digitalWrite(vibration, LOW); 
    _delay_ms(500); 
      
  //int range = pmax - pmin; // pressure range 
  range =  (pmax - pmin) / step; 
  Serial.println("range");
  Serial.println(range); 
  _delay_ms(2000);   
 }


void loop() {
  // put your main code here, to run repeatedly:
    myservo.write(servomotor_positioning()); // motor control
   
    int c_curr = analogRead(s_current);
    if (c_curr > mean_current+80) { // to set in function of the specific current sensor used in the experimentation and the chosen threshold
    digitalWrite(vibration, HIGH);
    }
    else {
    digitalWrite(vibration, LOW); 
    }
                  
   _delay_ms(50);                      
}
